
#include <Wire.h>
#include "RTClib.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include "LowPower.h"

boolean LEDHBit0;
boolean LEDHBit1;
boolean LEDHBit2;
boolean LEDHBit3;
boolean LEDMBit0;
boolean LEDMBit1;
boolean LEDMBit2;
boolean LEDMBit3;
boolean LEDMBit4;
boolean LEDMBit5;
boolean ButtonPressDebActive;
int toggleflag;
int ButtonPressTime;    //o: not pressed, 1: short Press <= 3 sek, 2: Long Press > 3sek
float BatVoltage;           //2.8V min bis 3.6V max Nominal LIR 1220


#define BatStatVoltage100perc 4.0
#define BatStatVoltage75perc 3.8
#define BatStatVoltage50perc 3.6
#define BatStatVoltage25perc 3.5

//Pin Definitionen Arduini Mini Pro (168, 3.3V, 8MHz)
#define LEDDatumPin     4 
#define LED_PM_Pin      5
#define LEDHBit0Pin     9
#define LEDHBit1Pin     8
#define LEDHBit2Pin     7
#define LEDHBit3Pin     6
#define LEDMBit0Pin     10
#define LEDMBit1Pin     12    //MISO
#define LEDMBit2Pin     13
#define LEDMBit3Pin     A0
#define LEDMBit4Pin     A1
#define LEDMBit5Pin     A2
#define ButtonPressPin  A3    //Pin für den Taster
#define ADC_PIN	        A6    //ADC2, Batterie Stand Überwachung, A6 ist nur Analog Eingangs Pin, bei Anpassung mus das Register ADMUX angepasst werden
#define RTC_Vcc_PIN   11    //D11 , MOSI, Einschalten des RTC zum Schreiben/Lesen der Zeitinformation

//Timeout & Entprellung
#define ButtonLongPressTimeThreshold 60 //x30mSek
#define StateTimeOut 150 //x30mSek (30 = 1 sek)
#define ButtonDebounceTime 2 //x30mSek

#define START_TIMER0 TCCR0B |= (1 << CS02) | (1 << CS00)     // 1024 als Prescale-Wert spezifizieren
#define STOP_TIMER0  TCCR0B &= 0b11111000
#define CLEAR_TIMER0 TCNT0 = 0


uint8_t timer30ms;
uint8_t DebounceCounter;

uint8_t MinuteNeu = 0;
uint8_t StundeNeu = 0;
uint8_t TagNeu = 0;
uint8_t MonatNeu = 0;

uint16_t Jahr = 2019;
uint8_t Monat = 1;
uint8_t Tag = 1;
uint8_t Stunde = 11;
uint8_t Minute = 0;
uint8_t Sekunde = 0;

int Status = 0; 
//0: Init/Sleep
//1: BatAnzeige
//10: Zeitabzeige, 
//11: Zeitanzeige einstellen Minute
//12: Zeitanzeige einstellen Stunde 
//20: Datumsanzeige
//21: Datumsanzeige einstellen Tag
//22: Datumsanzeige einstellen Monat

RTC_DS1307 rtc;


void setup(){
	
	//***I/Os INIT**
  pinMode(LEDHBit0Pin, OUTPUT); //D13 ist output LEDHBit0
  pinMode(LEDHBit1Pin, OUTPUT); //D13 ist output LEDHBit1
  pinMode(LEDHBit2Pin, OUTPUT); //D12 ist output LEDHBit2
  pinMode(LEDHBit3Pin, OUTPUT); //D13 ist output LEDHBit3
  
  pinMode(LEDMBit0Pin, OUTPUT); //D13 ist output LEDMBit0
  pinMode(LEDMBit1Pin, OUTPUT); //D13 ist output LEDMBit1
  pinMode(LEDMBit2Pin, OUTPUT); //D12 ist output LEDMBit2
  pinMode(LEDMBit3Pin, OUTPUT); //D13 ist output LEDMBit3
  pinMode(LEDMBit4Pin, OUTPUT); //D12 ist output LEDMBit4
  pinMode(LEDMBit5Pin, OUTPUT); //D13 ist output LEDMBit5
  pinMode(LEDDatumPin, OUTPUT);
  pinMode(LED_PM_Pin, OUTPUT);
  pinMode(RTC_Vcc_PIN, OUTPUT);// Versorgung vom RTC
  pinMode(LEDDatumPin, OUTPUT); //D2 Status LED Datum/Uhrzeit
  pinMode(ADC_PIN,INPUT);
  pinMode(ButtonPressPin, INPUT);
  pinMode(ButtonPressPin, INPUT_PULLUP);

  	//***SERIAL INIT***
    while (!Serial); // for Leonardo/Micro/Zero
 
    Serial.begin(9600);
}

void init_interupts()
{

    Serial.println("Start");  
    digitalWrite(RTC_Vcc_PIN, HIGH);
    delay(100);
    Serial.println("Test");
    delay(100);

     
    if (! rtc.begin()) {
    Serial.println("Couldn't find RTC");
    delay(100);
    while (1);
    }
    
    
    if (! rtc.isrunning()) {
    Serial.println("RTC is has no Starting Time!");
          //rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
    rtc.adjust(DateTime (2018, 5, 1,11, 11, 11));     //Start Zeit setzen
          //DateTime (uint16_t year, uint8_t month, uint8_t day,uint8_t hour = 0, uint8_t min = 0, uint8_t sec = 0);
    }
    Serial.println("RTC adjusted!");
    delay(100);
  
  	//***TIMER für RTC INIT***
  
  OCR0A = 0x9C;    //8bit Register, for 8MHz, 0x3D08 für  16Mhz Clock (mode 4)   Vergleichsregisterwert 1 (1sek)  0x7A12 = dez 31250 , 8.000.000 / 256 =  31250, 3125dez = 0x0C35

  Serial.print(TCCR0A);
  TCCR0A |= (1 << WGM01);   // Mode 4, CTC on OCR1A
  Serial.print(TCCR0A);
  TIMSK0 |= (1 << OCIE0A);  //Set interrupt on compare match
  STOP_TIMER0;             // this code sets up timer1 for a 1s  @ 16Mhz Clock (mode 4)
  // set prescaler to 256 and start the timer

  //***DIGITAL INTERRUPT INIT***

    PCICR |= (1 << PCIE1);    // set PCIE1 to enable PCMSK1 scan, PCINT11 -> PC3, ADC3
    PCMSK1|= (1 << PCINT11);  // set PCINT19 to trigger an interrupt on state change
	
  //* ADC zur Batteriespannungsüberwachung*/
    
	ADCSRA |= (1 << ADEN);		// Enable the ADC
  ADCSRA |= (1 << ADSC);    // Perform Single AD Conversation
					          // and set the ACD clock pre-scalar to clk/128
	ADMUX = 0xC6;			// ADC6, Select internal 1.1V as Vref, right justify -> 10bit, Vref = 1.1
					          // data registers and select ADC6 as input channel
  sei();            // enable interrupts


//SLEEP MODES KONFIGURIEREN

  
  //set_sleep_mode(SLEEP_MODE_PWR_DOWN);//DOWN
  //sleep_enable();
}

void LED_PM_Off() {
    digitalWrite(LED_PM_Pin, HIGH);   //Datum LED aus, Logik invertiert
}
void LED_Dat_Off() {
    digitalWrite(LEDDatumPin, HIGH);   //Datum LED aus, Logik invertiert
}

void LED_PM_On() {
    digitalWrite(LED_PM_Pin, LOW);   //Datum LED aus, Logik invertiert
}
void LED_Dat_On() {
    digitalWrite(LEDDatumPin, LOW);   //Datum LED aus, Logik invertiert
}

void LEDsTimeDateOff() {
    digitalWrite(LEDHBit0Pin, HIGH);
    digitalWrite(LEDHBit1Pin, HIGH);
    digitalWrite(LEDHBit2Pin, HIGH);
    digitalWrite(LEDHBit3Pin, HIGH);
    digitalWrite(LEDMBit0Pin, HIGH);
    digitalWrite(LEDMBit1Pin, HIGH);
    digitalWrite(LEDMBit2Pin, HIGH);
    digitalWrite(LEDMBit3Pin, HIGH);
    digitalWrite(LEDMBit4Pin, HIGH);
    digitalWrite(LEDMBit5Pin, HIGH);
}

void LEDsOff() {
    LEDsTimeDateOff();
    LED_PM_Off();
    LED_Dat_Off();
}

void TimeOut(){

STOP_TIMER0;
CLEAR_TIMER0;
LEDsOff();
digitalWrite(RTC_Vcc_PIN, LOW);
//LowPower.idle(SLEEP_FOREVER, ADC_OFF, TIMER2_OFF,TIMER1_OFF, TIMER0_OFF, SPI_OFF,USART0_OFF, TWI_OFF);
LowPower.powerDown(SLEEP_FOREVER, ADC_OFF, BOD_OFF);

}

void ReadCurrentDateTimeFromDS1307() {
    digitalWrite(RTC_Vcc_PIN, HIGH);
    DateTime now = rtc.now();
    Stunde = now.hour();
    Minute = now.minute();
    Sekunde = now.second();  
    Monat =  now.month();
    Tag = now.day();
}

void ResetButtonTimer() {
timer30ms = 0;
ButtonPressTime = 0;
}

void LEDsShowTime() {
    if (Stunde > 12)
    {
    if ((Stunde-12) & 0x01)
    {digitalWrite(LEDHBit0Pin, LOW);}
    else
    {digitalWrite(LEDHBit0Pin, HIGH);}
    if ((Stunde-12) & 0x02)
    {digitalWrite(LEDHBit1Pin, LOW);}
    else
    {digitalWrite(LEDHBit1Pin, HIGH);}
    if ((Stunde-12) & 0x04)
    {digitalWrite(LEDHBit2Pin, LOW);}
    else
    {digitalWrite(LEDHBit2Pin, HIGH);}
    if ((Stunde-12) & 0x08)
    {digitalWrite(LEDHBit3Pin, LOW);}
    else
    {digitalWrite(LEDHBit3Pin, HIGH);}
	  LED_PM_On();
    }
    else {
    if (Stunde & 0x01)
    {digitalWrite(LEDHBit0Pin, LOW);}
    else
    {digitalWrite(LEDHBit0Pin, HIGH);}
    if (Stunde & 0x02)
    {digitalWrite(LEDHBit1Pin, LOW);}
    else
    {digitalWrite(LEDHBit1Pin, HIGH);}
    if (Stunde & 0x04)
    {digitalWrite(LEDHBit2Pin, LOW);}
    else
    {digitalWrite(LEDHBit2Pin, HIGH);}
    if (Stunde & 0x08)
    {digitalWrite(LEDHBit3Pin, LOW);}
    else
    {digitalWrite(LEDHBit3Pin, HIGH);}
	  LED_PM_Off();
    }

    if (Minute & 0x01)
    {digitalWrite(LEDMBit0Pin, LOW);}
    else
    {digitalWrite(LEDMBit0Pin, HIGH);}
    if (Minute & 0x02)
    {digitalWrite(LEDMBit1Pin, LOW);}
    else
    {digitalWrite(LEDMBit1Pin, HIGH);}
    if (Minute & 0x04)
    {digitalWrite(LEDMBit2Pin, LOW);}
    else
    {digitalWrite(LEDMBit2Pin, HIGH);}
    if (Minute & 0x08)
    {digitalWrite(LEDMBit3Pin, LOW);}
    else
    {digitalWrite(LEDMBit3Pin, HIGH);}
    if (Minute & 0x10)
    {digitalWrite(LEDMBit4Pin, LOW);}
    else
    {digitalWrite(LEDMBit4Pin, HIGH);}
    if (Minute & 0x20)
    {digitalWrite(LEDMBit5Pin, LOW);}
    else
    {digitalWrite(LEDMBit5Pin, HIGH);}
       
    LED_Dat_Off();
}

void LEDsChangeMinute() {  
    LED_Dat_Off();
    LEDsTimeDateOff();
    if (toggleflag == 0) {      //500ms
    if (MinuteNeu & 0x01)
    {digitalWrite(LEDMBit0Pin, LOW);}
    else
    {digitalWrite(LEDMBit0Pin, HIGH);}
    if (MinuteNeu & 0x02)
    {digitalWrite(LEDMBit1Pin, LOW);}
    else
    {digitalWrite(LEDMBit1Pin, HIGH);}
    if (MinuteNeu & 0x04)
    {digitalWrite(LEDMBit2Pin, LOW);}
    else
    {digitalWrite(LEDMBit2Pin, HIGH);}
    if (MinuteNeu & 0x08)
    {digitalWrite(LEDMBit3Pin, LOW);}
    else
    {digitalWrite(LEDMBit3Pin, HIGH);}
    if (MinuteNeu & 0x10)
    {digitalWrite(LEDMBit4Pin, LOW);}
    else
    {digitalWrite(LEDMBit4Pin, HIGH);}
    if (MinuteNeu & 0x20)
    {digitalWrite(LEDMBit5Pin, LOW);}
    else
    {digitalWrite(LEDMBit5Pin, HIGH);}
    }
    else{
    digitalWrite(LEDMBit0Pin, HIGH);
    digitalWrite(LEDMBit1Pin, HIGH);
    digitalWrite(LEDMBit2Pin, HIGH);
    digitalWrite(LEDMBit3Pin, HIGH);
    digitalWrite(LEDMBit4Pin, HIGH);
    digitalWrite(LEDMBit5Pin, HIGH);
    }
}

void LEDsChangeStunde() {
    LED_Dat_Off();
    LEDsTimeDateOff();
if (toggleflag == 0){      //500ms
    if (StundeNeu > 12)
    {
    if ((StundeNeu-12) & 0x01)
    {digitalWrite(LEDHBit0Pin, LOW);}
    else
    {digitalWrite(LEDHBit0Pin, HIGH);}
    if ((StundeNeu-12) & 0x02)
    {digitalWrite(LEDHBit1Pin, LOW);}
    else
    {digitalWrite(LEDHBit1Pin, HIGH);}
    if ((StundeNeu-12) & 0x04)
    {digitalWrite(LEDHBit2Pin, LOW);}
    else
    {digitalWrite(LEDHBit2Pin, HIGH);}
    if ((StundeNeu-12) & 0x08)
    {digitalWrite(LEDHBit3Pin, LOW);}
    else
    {digitalWrite(LEDHBit3Pin, HIGH);}
	  LED_PM_On();
    }
    else {
    if (StundeNeu & 0x01)
    {digitalWrite(LEDHBit0Pin, LOW);}
    else
    {digitalWrite(LEDHBit0Pin, HIGH);}
    if (StundeNeu & 0x02)
    {digitalWrite(LEDHBit1Pin, LOW);}
    else
    {digitalWrite(LEDHBit1Pin, HIGH);}
    if (StundeNeu & 0x04)
    {digitalWrite(LEDHBit2Pin, LOW);}
    else
    {digitalWrite(LEDHBit2Pin, HIGH);}
    if (StundeNeu & 0x08)
    {digitalWrite(LEDHBit3Pin, LOW);}
    else
    {digitalWrite(LEDHBit3Pin, HIGH);}
	  LED_PM_Off();
    }
}}

void LEDsShowBatStatus() {
if (toggleflag == 0){      //500ms
    if (BatVoltage >= BatStatVoltage25perc){
    digitalWrite(LEDHBit0Pin, LOW);}
    if (BatVoltage >= BatStatVoltage50perc){
    digitalWrite(LEDHBit1Pin, LOW);}
    if (BatVoltage >= BatStatVoltage75perc){
    digitalWrite(LEDHBit2Pin, LOW);}
    if (BatVoltage >= BatStatVoltage100perc){
    digitalWrite(LEDHBit3Pin, LOW);}
    }
else {LEDsOff();}
}

void LEDsChangeMonat() {
    LED_Dat_On();
    LED_PM_Off();
    LEDsTimeDateOff();
if (toggleflag == 0){      //500ms

    if (MonatNeu & 0x01)
    {digitalWrite(LEDHBit0Pin, LOW);}
    else
    {digitalWrite(LEDHBit0Pin, HIGH);}
    if (MonatNeu & 0x02)
    {digitalWrite(LEDHBit1Pin, LOW);}
    else
    {digitalWrite(LEDHBit1Pin, HIGH);}
    if (MonatNeu & 0x04)
    {digitalWrite(LEDHBit2Pin, LOW);}
    else
    {digitalWrite(LEDHBit2Pin, HIGH);}
    if (MonatNeu & 0x08)
    {digitalWrite(LEDHBit3Pin, LOW);}
    else
    {digitalWrite(LEDHBit3Pin, HIGH);}
    
}
else{
    digitalWrite(LEDHBit0Pin, HIGH);
    digitalWrite(LEDHBit1Pin, HIGH);
    digitalWrite(LEDHBit2Pin, HIGH);
    digitalWrite(LEDHBit3Pin, HIGH);
    }
}

void LEDsChangeTag() {  
    LED_Dat_On();
    LED_PM_Off();
    LEDsTimeDateOff();
    if (toggleflag == 0) {      //500ms
    if (TagNeu & 0x01)
    {digitalWrite(LEDMBit0Pin, LOW);}
    else
    {digitalWrite(LEDMBit0Pin, HIGH);}
    if (TagNeu & 0x02)
    {digitalWrite(LEDMBit1Pin, LOW);}
    else
    {digitalWrite(LEDMBit1Pin, HIGH);}
    if (TagNeu & 0x04)
    {digitalWrite(LEDMBit2Pin, LOW);}
    else
    {digitalWrite(LEDMBit2Pin, HIGH);}
    if (TagNeu & 0x08)
    {digitalWrite(LEDMBit3Pin, LOW);}
    else
    {digitalWrite(LEDMBit3Pin, HIGH);}
    if (TagNeu & 0x10)
    {digitalWrite(LEDMBit4Pin, LOW);}
    else
    {digitalWrite(LEDMBit4Pin, HIGH);}
    if (TagNeu & 0x20)
    {digitalWrite(LEDMBit5Pin, LOW);}
    else
    {digitalWrite(LEDMBit5Pin, HIGH);}
    }
    else{
    digitalWrite(LEDMBit0Pin, HIGH);
    digitalWrite(LEDMBit1Pin, HIGH);
    digitalWrite(LEDMBit2Pin, HIGH);
    digitalWrite(LEDMBit3Pin, HIGH);
    digitalWrite(LEDMBit4Pin, HIGH);
    digitalWrite(LEDMBit5Pin, HIGH);
    }
}

void LEDsShowDate() {
    LED_PM_Off();
    if (Monat & 0x01)
    {digitalWrite(LEDHBit0Pin, LOW);}
    else
    {digitalWrite(LEDHBit0Pin, HIGH);}
    if (Monat & 0x02)
    {digitalWrite(LEDHBit1Pin, LOW);}
    else
    {digitalWrite(LEDHBit1Pin, HIGH);}
    if (Monat & 0x04)
    {digitalWrite(LEDHBit2Pin, LOW);}
    else
    {digitalWrite(LEDHBit2Pin, HIGH);}
    if (Monat & 0x08)
    {digitalWrite(LEDHBit3Pin, LOW);}
    else
    {digitalWrite(LEDHBit3Pin, HIGH);}

    if (Tag & 0x01)
    {digitalWrite(LEDMBit0Pin, LOW);}
    else
    {digitalWrite(LEDMBit0Pin, HIGH);}
    if (Tag & 0x02)
    {digitalWrite(LEDMBit1Pin, LOW);}
    else
    {digitalWrite(LEDMBit1Pin, HIGH);}
    if (Tag & 0x04)
    {digitalWrite(LEDMBit2Pin, LOW);}
    else
    {digitalWrite(LEDMBit2Pin, HIGH);}
    if (Tag & 0x08)
    {digitalWrite(LEDMBit3Pin, LOW);}
    else
    {digitalWrite(LEDMBit3Pin, HIGH);}
    if (Tag & 0x10)
    {digitalWrite(LEDMBit4Pin, LOW);}
    else
    {digitalWrite(LEDMBit4Pin, HIGH);}
    if (Tag & 0x20)
    {digitalWrite(LEDMBit5Pin, LOW);}
    else
    {digitalWrite(LEDMBit5Pin, HIGH);}

    LED_Dat_On();
}

	//STATUSMASCHINE////////////////////////////////////////
  
void StatusMaschine()
{
  if (Status > 0){digitalWrite(RTC_Vcc_PIN, HIGH);}

  switch(Status) {
    case 0:			//Standby
	LEDsOff();
	if (ButtonPressTime  == 1)
        {
        ResetButtonTimer();
        Status = 10;
        ReadCurrentDateTimeFromDS1307();
        Serial.print("_Ti:");
        Serial.print(Stunde);
        Serial.print(":");
        Serial.print(Minute);
        }
    break;

  case 1:      //Bat Anzeige
  LEDsShowBatStatus();
  if (ButtonPressTime  == 1 || timer30ms > StateTimeOut) //Zurück in Standby
        {
        ResetButtonTimer();
        Status = 0;   //Zurück in Standby
        //Serial.print("_TimerStp_");
        TimeOut();
        }
    break;


  case 10:			//Zeitanzeige
  
	LEDsShowTime();
	if (ButtonPressTime  == 1)
        {
        ResetButtonTimer();
        Status = 20;			// Zur Datumsanzeige
        }
  if (ButtonPressTime == 2)
        {
        ResetButtonTimer();
        Status = 11;			// Zur Datumsanzeige
        MinuteNeu = Minute;
        }
	if (timer30ms > StateTimeOut && ButtonPressDebActive == false)			//Zurück in Standby
        {
        Status = 0;
        ResetButtonTimer();
        Serial.print("_TimerStp_");
        TimeOut();
        }
    break;
	
	case 11:			//11: Zeitanzeige einstellen Minute
  
  LEDsChangeMinute();
        
        if (ButtonPressTime  == 1)      //kurz drücken
        {
        ResetButtonTimer();
        if (MinuteNeu == 59){
        MinuteNeu = 0;}
        else{
        MinuteNeu++;}			              // Minute inklementieren
        }

        if (timer30ms > StateTimeOut && ButtonPressDebActive == false)  //Timeout Zurück zu Zeit
        {
        Status = 10;
        ResetButtonTimer();
        }

        if (ButtonPressTime == 2)  //Minute speichern Speichern, zur Stunde Änderung
        {
        ResetButtonTimer();
        rtc.adjust(DateTime (Jahr, Monat, Tag, Stunde, MinuteNeu, Sekunde));     //Minute neu setzen, zur Stunde setzen
        Status = 12;
        Serial.print(MinuteNeu);
        Minute = MinuteNeu;
        StundeNeu = Stunde;
        }
    break;
	
	case 12:			//12: Zeitanzeige einstellen Stunde
  LEDsChangeStunde();
        if (ButtonPressTime  == 1)      //kurz drücken
        {
        ResetButtonTimer();
        if (StundeNeu == 23){
        StundeNeu = 0;}
        else{
        StundeNeu++;}			// Stunde inklementieren
        }
        if (timer30ms > StateTimeOut && ButtonPressDebActive == false)	        //Timeout Zurück zu Zeit
        {
        Status = 10;
        ResetButtonTimer();
        }
        if (ButtonPressTime == 2)	//Stunde speichern zurück zur Zeit
        {
        ResetButtonTimer();
        rtc.adjust(DateTime (Jahr, Monat, Tag, StundeNeu, Minute, Sekunde));     //Stunde neu gesetzt, zurück zur Zeit
        Stunde = StundeNeu;
        Status = 10;
        }
    break;
	
	case 20:			//20: Datumsanzeige     
	LEDsShowDate();
	if (ButtonPressTime  == 1) //Zu Bat Anzeige
        {
        LEDsOff();
        ResetButtonTimer();
        Status = 1;		//Zurück in Standby
        
        }
  if (timer30ms > StateTimeOut) //Zurück in Standby
        {
        ResetButtonTimer();
        Status = 0;   //Zurück in Standby
        TimeOut();
        }
  if (ButtonPressTime == 2)  //zum Tag anpassen
        {
        ResetButtonTimer();
        Status = 21;
        TagNeu = Tag;
   }
    break;
	
	case 21:			//21: Datumsanzeige einstellen Tag
  LEDsChangeTag();
     if (ButtonPressTime  == 1)      //kurz drücken
        {
        ResetButtonTimer();
        switch (Monat){
        case 1: case 3: case 5: case 7: case 8: case 10: case 12:
            if (TagNeu == 31)
            {TagNeu = 1;}
            else 
            {TagNeu++;
            Serial.print(TagNeu);     //TEST
            }
            break;
        case 4: case 6: case 9: case 11:
            if (TagNeu == 30)
            {TagNeu = 1;}
            else 
            {TagNeu++;}
            break;
        case 2:
            if (TagNeu == 28)
            {TagNeu = 1;}
            else 
            {TagNeu++;}
            break;  
        }}
        if (ButtonPressTime == 2)  //Tag speichern Speichern, zur Monat Änderung
        {
        ResetButtonTimer();
        rtc.adjust(DateTime (Jahr, Monat, TagNeu, Stunde, Minute, Sekunde));     //Tag neu gesetz, zum Monat setzen
        Status = 22;
        Tag = TagNeu;
        MonatNeu = Monat;
        }
        
        if (timer30ms > StateTimeOut && ButtonPressDebActive == false)  //Timeout Zurück zu Zeit
        {
        ResetButtonTimer();
        Status = 0;
        TimeOut();
        }
    break;
	
	case 22:			//22: Datumsanzeige einstellen Monat
    LEDsChangeMonat();
        if (ButtonPressTime  == 1)      //kurz drücken
        {
        ButtonPressTime = 0;
        if (MonatNeu == 12){
        MonatNeu = 1;}
        else{
        MonatNeu++;}      // Stunde inklementieren
        }
        if (timer30ms > StateTimeOut && ButtonPressDebActive == false)         //Timeout Zurück zu Datum
        {
        Status = 20;
        Serial.print("_20_");
        timer30ms = 0;
        }
        if (ButtonPressTime == 2) //Stunde speichern zurück zum Datum
        {
        ResetButtonTimer();
        rtc.adjust(DateTime (Jahr, MonatNeu, Tag, Stunde, Minute, Sekunde));     //Zeit neu setzen, zurück zum datum
        Monat = MonatNeu;
        Status = 20;
        Serial.print("_20_");
        }
    break;
	
  }

}

void loop () 
{
  init_interupts();
  LEDsOff();
  LowPower.powerDown(SLEEP_FOREVER, ADC_OFF, BOD_OFF);

  while( 1 ) {  
  StatusMaschine();
  }
}
 
//INTERRUPT ROUTINEN       

ISR (TIMER0_COMPA_vect)  //Timer/Counter1 Compare Match A
{
timer30ms++;
    if (timer30ms % 10 == 0)
    {
      toggleflag ^= 1;
    }
    if (ButtonPressDebActive == false && DebounceCounter > 0)
    { DebounceCounter--;}
    else
    { DebounceCounter++;}
    if(DebounceCounter > ButtonLongPressTimeThreshold)
    {
        Serial.print("LngBut_");
        ButtonPressDebActive = false;
        DebounceCounter = 0;
        ButtonPressTime = 2;
        timer30ms = 0;
    }
}

//https://sites.google.com/site/qeewiki/books/avr-guide/external-interrupts-on-the-atmega328
ISR (PCINT1_vect)  //Pin Change Interrupt Request 2 (pins D0 to D7) 
{   
    if(digitalRead(ButtonPressPin) == 0)
    {   
        sleep_disable();          // Sleep Mode beenden
        digitalWrite(RTC_Vcc_PIN, HIGH);
          ADCSRA |= (1 << ADEN);    // Enable the ADC
          ADCSRA |= (1 << ADSC);    // Perform Single AD Conversation
        ButtonPressDebActive = true;
        START_TIMER0;             // this code sets up timer1
        Serial.print("_But_"); 
        
    }
    if(digitalRead(ButtonPressPin) == 1 && DebounceCounter > ButtonDebounceTime && DebounceCounter <= ButtonLongPressTimeThreshold)
    {
        //Serial.print("SrtBut_");
        ButtonPressDebActive = false;
        DebounceCounter = 0;
        timer30ms = 0;
        ButtonPressTime = 1;
        BatVoltage= (((ADCH*256+ADCL) * 1.1 / 1024)/510)*2010;  //Vref = 1.1, right Justifyed, Spannungsteiler, 510k und 1500k
        Serial.print("U:");
        Serial.println(BatVoltage);
        ADCSRA &= ~(1 << ADEN);   //ADC Abschalten

    }
    
}
